#! /bin/sh
#
# chksum.sh - add check-sum section into vmlinux elf file
#
# Copyright (C) 2001  Sony Computer Entertainment Inc.
# Copyright 2001  Sony Corportaion.
#
# This file is subject to the terms and conditions of the GNU General Public
# License.  See the file "COPYING" in the main directory of this archive
# for more details.
#

tmpzero=zeros.$$
tmplinux=vmlinux.$$
tmpsum=sum.$$

objcpy=${CROSS_COMPILE}objcopy

print_usage()
{
	echo
	echo Usage:
	echo "$cmdname	[infile [outfile]]"
	echo
}

rm_tmpfiles()
{
	rm -f $tmplinux $tmpzero $tmpsum
}

error()
{
	rm_tmpfiles
	exit 1
}

cmdname=`basename $0`

if [ \( $# -gt 0 \) -a  \( -r "$1" \) ]; then
    orglinux=$1
    shift
    if [  $# -gt 0  ]; then
        sumlinux=$1
    else
        sumlinux=${orglinux}.sum
    fi
else
    print_usage
    exit 1;
fi


trap error 1

cp -p $orglinux $tmplinux || error

#
# calc. crc check sum
#
sum=`cksum $tmplinux | awk '{printf "0x%08x\n",$1}'` || error

#
# add crc value(big-endian) to the end of file
#
echo $sum | xxd -r > $tmpsum || error
cat $orglinux $tmpsum >$tmplinux

mv $tmplinux $sumlinux || error

rm_tmpfiles

echo  "  ${cmdname}: Adding checksum of $orglinux (CRC=$sum)"
echo  "              into $sumlinux."

exit 0
