/*
 * MIPS specific syscalls
 *
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1995, 1996, 1997 by Ralf Baechle
 *
 * $Id: sysmips.c,v 1.6 1998/08/25 09:14:42 ralf Exp $
 */
#include <linux/errno.h>
#include <linux/linkage.h>
#include <linux/mm.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
#include <linux/sched.h>
#include <linux/string.h>
#include <linux/utsname.h>

#include <asm/cachectl.h>
#include <asm/pgtable.h>
#include <asm/sysmips.h>
#include <asm/uaccess.h>
#ifdef CONFIG_CPU_R5900
#include <asm/sys_r5900.h>
#endif

/*
 * How long a hostname can we get from user space?
 *  -EFAULT if invalid area or too long
 *  0 if ok
 *  >0 EFAULT after xx bytes
 */
static inline int
get_max_hostname(unsigned long address)
{
	struct vm_area_struct * vma;

	vma = find_vma(current->mm, address);
	if (!vma || vma->vm_start > address || !(vma->vm_flags & VM_READ))
		return -EFAULT;
	address = vma->vm_end - address;
	if (address > PAGE_SIZE)
		return 0;
	if (vma->vm_next && vma->vm_next->vm_start == vma->vm_end &&
	   (vma->vm_next->vm_flags & VM_READ))
		return 0;
	return address;
}

asmlinkage int
sys_sysmips(int cmd, int arg1, int arg2, int arg3)
{
	int	*p;
	char	*name;
	int	flags, tmp, len, retval;

	lock_kernel();
	switch(cmd)
	{
	case SETNAME:
		retval = -EPERM;
		if (!capable(CAP_SYS_ADMIN))
			goto out;

		name = (char *) arg1;
		len = strlen_user(name);

		retval = len;
		if (len < 0)
			goto out;

		retval = -EINVAL;
		if (len == 0 || len > __NEW_UTS_LEN)
			goto out;

		copy_from_user(system_utsname.nodename, name, len);
		system_utsname.nodename[len] = '\0';
		retval = 0;
		goto out;

	case MIPS_ATOMIC_SET: {
                int *ptr, val, ret, err, tmp;
                struct siginfo info;

                ptr = (int *)arg1;
                val = (int)arg2;

                /* Don't emulate unaligned accesses. */
                if ((int)ptr & 3) {
                        info.si_signo = SIGBUS;
                        info.si_code = BUS_ADRALN;
			ret = -EFAULT;
                        goto fault;
                }

                /* A zero here saves us three instructions. */
                err = verify_area(VERIFY_WRITE, ptr, 0);
                if (err) {
                        info.si_signo = SIGSEGV;
                        info.si_code = SEGV_ACCERR;
			ret = -EFAULT;
                        goto fault;
                }

#if (_MIPS_ISA >= _MIPS_ISA_MIPS2)
                __asm__(".set   mips2\n\t"
                        "1:\n\t"
                        "ll     %0,%5\n\t"
                        ".set   push\n\t"
                        ".set   noreorder\n\t"
                        "beq    %0,%4,3f\n\t"
                        " move  %3,%4\n"
                        ".set   pop\n\t"
                        "2:\n\t"
                        "sc     %3,%1\n\t"
                        "beqz   %3,1b\n\t"
                        "3:\n\t"
                        ".set   mips0\n\t"
                        ".section .fixup,\"ax\"\n"
                        "4:\n\t"
                        "li     %2,%7\n\t"
                        "j      3b\n\t"
                        ".previous\n\t"
                        ".section __ex_table,\"a\"\n\t"
                        ".word  1b,4b\n\t"
                        ".word  2b,4b\n\t"
                        ".previous"
                        : "=&r" (ret), "=m" (*ptr), "=r" (err), "=&r" (tmp)
                        : "r" (val), "1" (*ptr), "2" (0), "i" (-EFAULT));
#else

		/* Accessing to *ptr for page-in. 
		 * We need to this, because access_ok() doesn't 
		 * actualy access to the *PTR. */
                err = __get_user(ret, ptr);
                if (err) {
                        info.si_signo = SIGSEGV;
                        info.si_code = SEGV_ACCERR;
                        goto fault;
                }

		/* 
		 * NOTE: The page could be paged-out here, if kernel is 
		 * premptable.
		 */

                save_and_cli(tmp);
                ret = *ptr;			/* We assume *ptr is in page */
                if (ret != val)
                  err = __put_user(val, ptr);    /* No fault
                                                   unless unwriteable. */
                restore_flags(tmp);
#endif

                if (err) {
                        info.si_signo = SIGSEGV;
                        info.si_code = SEGV_MAPERR;
                        goto fault;
                }

                if ( (ret < 0) && (ret >= -EMAXERRNO) ) {
                        info.si_signo = SIGSYS;
                        info.si_code = 0;
                        goto fault;
                }

                retval=ret;
		goto out;

fault:
                force_sig_info(info.si_signo, &info, current);
                retval=-EFAULT;
		goto out;
	}


	case MIPS_FIXADE:
		tmp = current->tss.mflags & ~3;
		current->tss.mflags = tmp | (arg1 & 3);
		retval = 0;
		goto out;

	case FLUSH_CACHE:
		flush_cache_all();
		retval = 0;
		goto out;

	case MIPS_RDNVRAM:
		retval = -EIO;
		goto out;

	case MIPS_TASKFLAGS:
		if (arg1 == 0) {
			int flags;
			flags = current->flags & (PF_TRACESYS|PF_PTRACED);
			if (__put_user(flags, (int *)arg2)) {
				retval = -EFAULT;
				goto out;
			}
		} else {
			current->flags &= ~(PF_TRACESYS|PF_PTRACED);
			current->flags |= (arg2 & (PF_TRACESYS|PF_PTRACED));
		}
		retval = 0;
		goto out;

#ifdef CONFIG_CPU_R5900
	case MIPS_SYS_R5900:
		retval = mips_sys_r5900(arg1, arg2, arg3);
		goto out;
#endif
	default:
		retval = -EINVAL;
		goto out;
	}

out:
	unlock_kernel();
	return retval;
}

/*
 * No implemented yet ...
 */
asmlinkage int
sys_cachectl(char *addr, int nbytes, int op)
{
	return -ENOSYS;
}

